@@ -1,7 +1,5 @@
 package Module::Build::Tiny;
-{
-  $Module::Build::Tiny::VERSION = '0.030';
-}
+$Module::Build::Tiny::VERSION = '0.031';
 use strict;
 use warnings;
 use Exporter 5.57 'import';
@@ -16,7 +14,7 @@ use File::Basename qw/basename dirname/;
 use File::Find ();
 use File::Path qw/mkpath rmtree/;
 use File::Spec::Functions qw/catfile catdir rel2abs abs2rel splitdir curdir/;
-use Getopt::Long qw/GetOptions/;
+use Getopt::Long qw/GetOptionsFromArray/;
 use JSON::PP 2 qw/encode_json decode_json/;
 
 sub write_file {
@@ -50,9 +48,9 @@ sub process_xs {
 	my ($source, $options) = @_;
 
 	die "Can't build xs files under --pureperl-only\n" if $options->{'pureperl-only'};
-	my (undef, @dirnames) = splitdir(dirname($source));
-	my $file_base = basename($source, '.xs');
-	my $archdir = catdir(qw/blib arch auto/, @dirnames, $file_base);
+	my (undef, @parts) = splitdir(dirname($source));
+	push @parts, my $file_base = basename($source, '.xs');
+	my $archdir = catdir(qw/blib arch auto/, @parts);
 	my $tempdir = 'temp';
 
 	my $c_file = catfile($tempdir, "$file_base.c");
@@ -65,9 +63,12 @@ sub process_xs {
 	my $builder = ExtUtils::CBuilder->new(config => $options->{config}->values_set);
 	my $ob_file = $builder->compile(source => $c_file, defines => { VERSION => qq/"$version"/, XS_VERSION => qq/"$version"/ }, include_dirs => [ curdir ]);
 
+	require DynaLoader;
+	my $mod2fname = defined &DynaLoader::mod2fname ? \&DynaLoader::mod2fname : sub { return $_[0][-1] };
+
 	mkpath($archdir, $options->{verbose}, oct '755') unless -d $archdir;
-	my $lib_file = catfile($archdir, "$file_base." . $options->{config}->get('dlext'));
-	return $builder->link(objects => $ob_file, lib_file => $lib_file, module_name => join '::', @dirnames, $file_base);
+	my $lib_file = catfile($archdir, $mod2fname->(\@parts) . '.' . $options->{config}->get('dlext'));
+	return $builder->link(objects => $ob_file, lib_file => $lib_file, module_name => join '::', @parts);
 }
 
 sub find {
@@ -121,8 +122,8 @@ my %actions = (
 sub Build {
 	my $action = @ARGV && $ARGV[0] =~ /\A\w+\z/ ? shift @ARGV : 'build';
 	die "No such action '$action'\n" if not $actions{$action};
-	unshift @ARGV, @{ decode_json(read_file('_build_params')) };
-	GetOptions(\my %opt, qw/install_base=s install_path=s% installdirs=s destdir=s prefix=s config=s% uninst:1 verbose:1 dry_run:1 pureperl-only:1 create_packlist=i/);
+	my($env, $bargv) = @{ decode_json(read_file('_build_params')) };
+	GetOptionsFromArray($_, \my %opt, qw/install_base=s install_path=s% installdirs=s destdir=s prefix=s config=s% uninst:1 verbose:1 dry_run:1 pureperl-only:1 create_packlist=i/) for ($env, $bargv, \@ARGV);
 	$_ = detildefy($_) for grep { defined } @opt{qw/install_base destdir prefix/}, values %{ $opt{install_path} };
 	@opt{ 'config', 'meta' } = (ExtUtils::Config->new($opt{config}), get_meta());
 	$actions{$action}->(%opt, install_paths => ExtUtils::InstallPaths->new(%opt, dist_name => $opt{meta}->name));
@@ -135,7 +136,7 @@ sub Build_PL {
 	write_file('Build', "#!perl\n$dir\nuse Module::Build::Tiny;\nBuild();\n");
 	make_executable('Build');
 	my @env = defined $ENV{PERL_MB_OPT} ? split_like_shell($ENV{PERL_MB_OPT}) : ();
-	write_file('_build_params', encode_json([ @env, @ARGV ]));
+	write_file('_build_params', encode_json([ \@env, \@ARGV ]));
 	$meta->save(@$_) for ['MYMETA.json'], [ 'MYMETA.yml' => { version => 1.4 } ];
 }
 
@@ -150,13 +151,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Module::Build::Tiny - A tiny replacement for Module::Build
 
 =head1 VERSION
 
-version 0.030
+version 0.031
 
 =head1 SYNOPSIS
 
